@@ -16,14 +16,14 @@
 
    You should have received a copy of the GNU General Public License
    along with Bash; see the file COPYING.  If not, write to the Free
-   Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
+   Software Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA. */
 
 #if !defined (_ALIAS_H_)
 #define _ALIAS_H_
 
-#include "hashlib.h"
+#include "stdc.h"
 
-extern char *xmalloc ();
+#include "hashlib.h"
 
 typedef struct alias {
   char *name;
@@ -38,33 +38,33 @@ typedef struct alias {
 /* The list of known aliases. */
 extern HASH_TABLE *aliases;
 
-extern void initialize_aliases ();
+extern void initialize_aliases __P((void));
 
 /* Scan the list of aliases looking for one with NAME.  Return NULL
    if the alias doesn't exist, else a pointer to the alias. */
-extern alias_t *find_alias ();
+extern alias_t *find_alias __P((char *));
 
 /* Return the value of the alias for NAME, or NULL if there is none. */
-extern char *get_alias_value ();
+extern char *get_alias_value __P((char *));
 
 /* Make a new alias from NAME and VALUE.  If NAME can be found,
    then replace its value. */
-extern void add_alias ();
+extern void add_alias __P((char *, char *));
 
 /* Remove the alias with name NAME from the alias list.  Returns
    the index of the removed alias, or -1 if the alias didn't exist. */
-extern int remove_alias ();
+extern int remove_alias __P((char *));
 
 /* Remove all aliases. */
-extern void delete_all_aliases ();
-
-/* Return a new line, with any aliases expanded. */
-extern char *alias_expand ();
+extern void delete_all_aliases __P((void));
 
 /* Return an array of all defined aliases. */
-extern alias_t **all_aliases ();
+extern alias_t **all_aliases __P((void));
 
 /* Expand a single word for aliases. */
-extern char *alias_expand_word ();
+extern char *alias_expand_word __P((char *));
+
+/* Return a new line, with any aliases expanded. */
+extern char *alias_expand __P((char *));
 
 #endif /* _ALIAS_H_ */
