@@ -41,13 +41,15 @@
 #define SPECIAL_BUILTIN 0x8	/* This is a Posix `special' builtin. */
 #define ASSIGNMENT_BUILTIN 0x10	/* This builtin takes assignment statements. */
 
+#define BASE_INDENT	4
+
 /* The thing that we build the array of builtins out of. */
 struct builtin {
   char *name;			/* The name that the user types. */
-  sh_builtin_func_t *function;		/* The address of the invoked function. */
+  sh_builtin_func_t *function;	/* The address of the invoked function. */
   int flags;			/* One of the #defines above. */
-  char **long_doc;		/* NULL terminated array of strings. */
-  char *short_doc;		/* Short version of documenation. */
+  char * const *long_doc;	/* NULL terminated array of strings. */
+  const char *short_doc;	/* Short version of documenation. */
   char *handle;			/* for future use */
 };
 
