@@ -50,7 +50,6 @@ static ARITH_COM *copy_arith_command __P((ARITH_COM *));
 static COND_COM *copy_cond_command __P((COND_COM *));
 #endif
 static SIMPLE_COM *copy_simple_command __P((SIMPLE_COM *));
-static FUNCTION_DEF *copy_function_def __P((FUNCTION_DEF *));
 
 WORD_DESC *
 copy_word (w)
@@ -58,13 +57,8 @@ copy_word (w)
 {
   WORD_DESC *new_word;
 
-  new_word = (WORD_DESC *)xmalloc (sizeof (WORD_DESC));
-#if 1
+  new_word = make_bare_word (w->word);
   new_word->flags = w->flags;
-#else
-  FASTCOPY ((char *)w, (char *)new_word, sizeof (WORD_DESC));
-#endif
-  new_word->word = savestring (w->word);
   return (new_word);
 }
 
@@ -74,15 +68,11 @@ WORD_LIST *
 copy_word_list (list)
      WORD_LIST *list;
 {
-  WORD_LIST *new_list, *temp;
+  WORD_LIST *new_list;
 
   for (new_list = (WORD_LIST *)NULL; list; list = list->next)
-    {
-      temp = (WORD_LIST *)xmalloc (sizeof (WORD_LIST));
-      temp->next = new_list;
-      new_list = temp;
-      new_list->word = copy_word (list->word);
-    }
+    new_list = make_word_list (copy_word (list->word), new_list);
+
   return (REVERSE_LIST (new_list, WORD_LIST *));
 }
 
@@ -128,6 +118,7 @@ copy_redirect (redirect)
     case r_deblank_reading_until:
       new_redirect->here_doc_eof = savestring (redirect->here_doc_eof);
       /*FALLTHROUGH*/
+    case r_reading_string:
     case r_appending_to:
     case r_output_direction:
     case r_input_direction:
@@ -137,10 +128,14 @@ copy_redirect (redirect)
     case r_output_force:
     case r_duplicating_input_word:
     case r_duplicating_output_word:
+    case r_move_input_word:
+    case r_move_output_word:
       new_redirect->redirectee.filename = copy_word (redirect->redirectee.filename);
       break;
     case r_duplicating_input:
     case r_duplicating_output:
+    case r_move_input:
+    case r_move_output:
     case r_close_this:
       break;
     }
@@ -170,6 +165,7 @@ copy_for_command (com)
 
   new_for = (FOR_COM *)xmalloc (sizeof (FOR_COM));
   new_for->flags = com->flags;
+  new_for->line = com->line;
   new_for->name = copy_word (com->name);
   new_for->map_list = copy_word_list (com->map_list);
   new_for->action = copy_command (com->action);
@@ -225,6 +221,7 @@ copy_case_command (com)
 
   new_case = (CASE_COM *)xmalloc (sizeof (CASE_COM));
   new_case->flags = com->flags;
+  new_case->line = com->line;
   new_case->word = copy_word (com->word);
   new_case->clauses = copy_case_clauses (com->clauses);
   return (new_case);
@@ -253,7 +250,7 @@ copy_if_command (com)
   new_if->flags = com->flags;
   new_if->test = copy_command (com->test);
   new_if->true_case = copy_command (com->true_case);
-  new_if->false_case = copy_command (com->false_case);
+  new_if->false_case = com->false_case ? copy_command (com->false_case) : com->false_case;
   return (new_if);
 }
 
@@ -301,22 +298,31 @@ copy_simple_command (com)
   new_simple = (SIMPLE_COM *)xmalloc (sizeof (SIMPLE_COM));
   new_simple->flags = com->flags;
   new_simple->words = copy_word_list (com->words);
-  new_simple->redirects = copy_redirects (com->redirects);
+  new_simple->redirects = com->redirects ? copy_redirects (com->redirects) : (REDIRECT *)NULL;
   new_simple->line = com->line;
   return (new_simple);
 }
 
-static FUNCTION_DEF *
+FUNCTION_DEF *
+copy_function_def_contents (old, new_def)
+     FUNCTION_DEF *old, *new_def;
+{
+  new_def->name = copy_word (old->name);
+  new_def->command = old->command ? copy_command (old->command) : old->command;
+  new_def->flags = old->flags;
+  new_def->line = old->line;
+  new_def->source_file = old->source_file ? savestring (old->source_file) : old->source_file;
+  return (new_def);
+}
+
+FUNCTION_DEF *
 copy_function_def (com)
      FUNCTION_DEF *com;
 {
   FUNCTION_DEF *new_def;
 
   new_def = (FUNCTION_DEF *)xmalloc (sizeof (FUNCTION_DEF));
-  new_def->name = copy_word (com->name);
-  new_def->command = copy_command (com->command);
-  new_def->flags = com->flags;
-  new_def->line = com->line;
+  new_def = copy_function_def_contents (com, new_def);
   return (new_def);
 }
 
