@@ -1,6 +1,6 @@
 /* mailcheck.c -- The check is in the mail... */
 
-/* Copyright (C) 1987,1989 Free Software Foundation, Inc.
+/* Copyright (C) 1987-2004 Free Software Foundation, Inc.
 
 This file is part of GNU Bash, the Bourne Again SHell.
 
@@ -29,17 +29,16 @@ Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA. */
 #if defined (HAVE_UNISTD_H)
 #  include <unistd.h>
 #endif
+#include "posixtime.h"
 #include "bashansi.h"
+#include "bashintl.h"
 
 #include "shell.h"
-#include "maxpath.h"
 #include "execute_cmd.h"
 #include "mailcheck.h"
 #include <tilde/tilde.h>
 
-#ifndef NOW
-#define NOW ((time_t)time ((time_t *)0))
-#endif
+extern int mailstat __P((const char *, struct stat *));
 
 typedef struct {
   char *name;
@@ -56,34 +55,34 @@ static FILEINFO **mailfiles = (FILEINFO **)NULL;
 static int mailfiles_count;
 
 /* The last known time that mail was checked. */
-static int last_time_mail_checked;
+static time_t last_time_mail_checked;
 
 /* Non-zero means warn if a mail file has been read since last checked. */
 int mail_warning;
 
+static int find_mail_file __P((char *));
+static void update_mail_file __P((int));
+static int add_mail_file __P((char *, char *));
+
+static int file_mod_date_changed __P((int));
+static int file_access_date_changed __P((int));
+static int file_has_grown __P((int));
+
+static char *parse_mailpath_spec __P((char *));
+
 /* Returns non-zero if it is time to check mail. */
 int
 time_to_check_mail ()
 {
   char *temp;
   time_t now;
-  long seconds;
+  intmax_t seconds;
 
   temp = get_string_value ("MAILCHECK");
-  seconds = -1L;
-
-  /* Skip leading whitespace in MAILCHECK. */
-  if (temp)
-    {
-      while (whitespace (*temp))
-	temp++;
-
-      seconds = atoi (temp);
-    }
 
   /* Negative number, or non-numbers (such as empty string) cause no
      checking to take place. */
-  if (seconds < 0)
+  if (temp == 0 || legal_number (temp, &seconds) == 0 || seconds < 0)
     return (0);
 
   now = NOW;
@@ -123,6 +122,15 @@ find_mail_file (file)
     } \
   while (0)
 
+#define UPDATE_MAIL_FILE(i, finfo) \
+  do \
+    { \
+      mailfiles[i]->access_time = finfo.st_atime; \
+      mailfiles[i]->mod_time = finfo.st_mtime; \
+      mailfiles[i]->file_size = finfo.st_size; \
+    } \
+  while (0)
+
 static void
 update_mail_file (i)
      int i;
@@ -131,12 +139,8 @@ update_mail_file (i)
   struct stat finfo;
 
   file = mailfiles[i]->name;
-  if (stat (file, &finfo) == 0)
-    {
-      mailfiles[i]->access_time = finfo.st_atime;
-      mailfiles[i]->mod_time = finfo.st_mtime;
-      mailfiles[i]->file_size = finfo.st_size;
-    }
+  if (mailstat (file, &finfo) == 0)
+    UPDATE_MAIL_FILE (i, finfo);
   else
     RESET_MAIL_FILE (i);
 }
@@ -155,12 +159,9 @@ add_mail_file (file, msg)
   i = find_mail_file (filename);
   if (i >= 0)
     {
-      if (stat (filename, &finfo) == 0)
-	{
-	  mailfiles[i]->mod_time = finfo.st_mtime;
-	  mailfiles[i]->access_time = finfo.st_atime;
-	  mailfiles[i]->file_size = finfo.st_size;
-	}
+      if (mailstat (filename, &finfo) == 0)
+	UPDATE_MAIL_FILE (i, finfo);
+
       free (filename);
       return i;
     }
@@ -183,9 +184,7 @@ reset_mail_files ()
   register int i;
 
   for (i = 0; i < mailfiles_count; i++)
-    {
-      RESET_MAIL_FILE (i);
-    }
+    RESET_MAIL_FILE (i);
 }
 
 /* Free the information that we have about the remembered mail files. */
@@ -209,7 +208,8 @@ free_mail_files ()
 }
 
 /* Return non-zero if FILE's mod date has changed and it has not been
-   accessed since modified. */
+   accessed since modified.  If the size has dropped to zero, reset
+   the cached mail file info. */
 static int
 file_mod_date_changed (i)
      int i;
@@ -221,9 +221,12 @@ file_mod_date_changed (i)
   file = mailfiles[i]->name;
   mtime = mailfiles[i]->mod_time;
 
-  if ((stat (file, &finfo) == 0) && (finfo.st_size > 0))
+  if ((mailstat (file, &finfo) == 0) && (finfo.st_size > 0))
     return (mtime != finfo.st_mtime);
 
+  if (finfo.st_size == 0 && mailfiles[i]->file_size > 0)
+    UPDATE_MAIL_FILE (i, finfo);
+
   return (0);
 }
 
@@ -239,7 +242,7 @@ file_access_date_changed (i)
   file = mailfiles[i]->name;
   atime = mailfiles[i]->access_time;
 
-  if ((stat (file, &finfo) == 0) && (finfo.st_size > 0))
+  if ((mailstat (file, &finfo) == 0) && (finfo.st_size > 0))
     return (atime != finfo.st_atime);
 
   return (0);
@@ -257,7 +260,7 @@ file_has_grown (i)
   file = mailfiles[i]->name;
   size = mailfiles[i]->file_size;
 
-  return ((stat (file, &finfo) == 0) && (finfo.st_size > size));
+  return ((mailstat (file, &finfo) == 0) && (finfo.st_size > size));
 }
 
 /* Take an element from $MAILPATH and return the portion from
@@ -291,14 +294,18 @@ parse_mailpath_spec (str)
 char *
 make_default_mailpath ()
 {
+#if defined (DEFAULT_MAIL_DIRECTORY)
   char *mp;
 
   get_current_user_info ();
-  mp = xmalloc (2 + sizeof (DEFAULT_MAIL_DIRECTORY) + strlen (current_user.user_name));
+  mp = (char *)xmalloc (2 + sizeof (DEFAULT_MAIL_DIRECTORY) + strlen (current_user.user_name));
   strcpy (mp, DEFAULT_MAIL_DIRECTORY);
   mp[sizeof(DEFAULT_MAIL_DIRECTORY) - 1] = '/';
   strcpy (mp + sizeof (DEFAULT_MAIL_DIRECTORY), current_user.user_name);
   return (mp);
+#else
+  return ((char *)NULL);
+#endif
 }
 
 /* Remember the dates of the files specified by MAILPATH, or if there is
@@ -323,8 +330,11 @@ remember_mail_dates ()
   if (mailpaths == 0)
     {
       mailpaths = make_default_mailpath ();
-      add_mail_file (mailpaths, (char *)NULL);
-      free (mailpaths);
+      if (mailpaths)
+	{
+	  add_mail_file (mailpaths, (char *)NULL);
+	  free (mailpaths);
+	}
       return;
     }
 
@@ -356,7 +366,6 @@ check_mail ()
   char *current_mail_file, *message;
   int i, use_user_notification;
   char *dollar_underscore, *temp;
-  WORD_LIST *tlist;
 
   dollar_underscore = get_string_value ("_");
   if (dollar_underscore)
@@ -374,9 +383,9 @@ check_mail ()
 	  int file_is_bigger;
 
 	  use_user_notification = mailfiles[i]->msg != (char *)NULL;
-	  message = mailfiles[i]->msg ? mailfiles[i]->msg : "You have mail in $_";
+	  message = mailfiles[i]->msg ? mailfiles[i]->msg : _("You have mail in $_");
 
-	  bind_variable ("_", current_mail_file);
+	  bind_variable ("_", current_mail_file, 0);
 
 #define atime mailfiles[i]->access_time
 #define mtime mailfiles[i]->mod_time
@@ -399,16 +408,14 @@ check_mail ()
 	  /* If the mod time is later than the access time and the file
 	     has grown, note the fact that this is *new* mail. */
 	  if (use_user_notification == 0 && (atime < mtime) && file_is_bigger)
-	    message = "You have new mail in $_";
+	    message = _("You have new mail in $_");
 #undef atime
 #undef mtime
 
-	  if ((tlist = expand_string (message, Q_DOUBLE_QUOTES)))
+	  if (temp = expand_string_to_string (message, Q_DOUBLE_QUOTES))
 	    {
-	      temp = string_list (tlist);
 	      puts (temp);
 	      free (temp);
-	      dispose_words (tlist);
 	    }
 	  else
 	    putchar ('\n');
@@ -417,13 +424,13 @@ check_mail ()
       if (mail_warning && file_access_date_changed (i))
 	{
 	  update_mail_file (i);
-	  printf ("The mail in %s has been read\n", current_mail_file);
+	  printf (_("The mail in %s has been read\n"), current_mail_file);
 	}
     }
 
   if (dollar_underscore)
     {
-      bind_variable ("_", dollar_underscore);
+      bind_variable ("_", dollar_underscore, 0);
       free (dollar_underscore);
     }
   else
