@@ -1,6 +1,6 @@
 /* shell.c -- GNU's idea of the POSIX shell specification. */
 
-/* Copyright (C) 1987,1991 Free Software Foundation, Inc.
+/* Copyright (C) 1987-2005 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
@@ -27,7 +27,7 @@
 #include "config.h"
 
 #include "bashtypes.h"
-#ifndef _MINIX
+#if !defined (_MINIX) && defined (HAVE_SYS_FILE_H)
 #  include <sys/file.h>
 #endif
 #include "posixstat.h"
@@ -43,6 +43,8 @@
 #  include <unistd.h>
 #endif
 
+#include "bashintl.h"
+
 #define NEED_SH_SETLINEBUF_DECL		/* used in externs.h */
 
 #include "shell.h"
@@ -60,6 +62,10 @@
 #include "execute_cmd.h"
 #include "findcmd.h"
 
+#if defined (USING_BASH_MALLOC) && defined (DEBUG) && !defined (DISABLE_MALLOC_WRAPPERS)
+#  include <malloc/shmalloc.h>
+#endif
+
 #if defined (HISTORY)
 #  include "bashhist.h"
 #  include <readline/history.h>
@@ -90,10 +96,11 @@ extern int shell_level;
 extern int subshell_environment;
 extern int last_command_exit_value;
 extern int line_number;
-extern char *primary_prompt, *secondary_prompt;
 extern int expand_aliases;
-extern char *this_command_name;
 extern int array_needs_making;
+extern int gnu_error_format;
+extern char *primary_prompt, *secondary_prompt;
+extern char *this_command_name;
 
 /* Non-zero means that this shell has already been run; i.e. you should
    call shell_reinitialize () if you need to start afresh. */
@@ -115,7 +122,7 @@ char *current_host_name = (char *)NULL;
    Specifically:
    0 = not login shell.
    1 = login shell from getty (or equivalent fake out)
-  -1 = login shell from "--login" flag.
+  -1 = login shell from "--login" (or -l) flag.
   -2 = both from getty, and from flag.
  */
 int login_shell = 0;
@@ -136,6 +143,7 @@ int hup_on_exit = 0;
 	0 = non-interactive shell script
 	1 = interactive
 	2 = -c command
+	3 = wordexp evaluation
    This is a superset of the information provided by interactive_shell.
 */
 int startup_state = 0;
@@ -187,11 +195,19 @@ static int do_version;			/* Display interesting version info. */
 static int make_login_shell;		/* Make this shell be a `-bash' shell. */
 static int want_initial_help;		/* --help option */
 
+int debugging_mode = 0;		/* In debugging mode with --debugger */
 int no_line_editing = 0;	/* Don't do fancy line editing. */
-int posixly_correct = 0;	/* Non-zero means posix.2 superset. */
 int dump_translatable_strings;	/* Dump strings in $"...", don't execute. */
 int dump_po_strings;		/* Dump strings in $"..." in po format */
 int wordexp_only = 0;		/* Do word expansion only */
+int protected_mode = 0;		/* No command substitution with --wordexp */
+
+#if defined (STRICT_POSIX)
+int posixly_correct = 1;	/* Non-zero means posix.2 superset. */
+#else
+int posixly_correct = 0;	/* Non-zero means posix.2 superset. */
+#endif
+
 
 /* Some long-winded argument names.  These are obviously new. */
 #define Int 1
@@ -203,6 +219,9 @@ struct {
   char **char_value;
 } long_args[] = {
   { "debug", Int, &debugging, (char **)0x0 },
+#if defined (DEBUGGER)
+  { "debugger", Int, &debugging_mode, (char **)0x0 },
+#endif
   { "dump-po-strings", Int, &dump_po_strings, (char **)0x0 },
   { "dump-strings", Int, &dump_translatable_strings, (char **)0x0 },
   { "help", Int, &want_initial_help, (char **)0x0 },
@@ -212,6 +231,7 @@ struct {
   { "noprofile", Int, &no_profile, (char **)0x0 },
   { "norc", Int, &no_rc, (char **)0x0 },
   { "posix", Int, &posixly_correct, (char **)0x0 },
+  { "protected", Int, &protected_mode, (char **)0x0 },
   { "rcfile", Charp, (int *)0x0, &bashrc_file },
 #if defined (RESTRICTED_SHELL)
   { "restricted", Int, &restricted, (char **)0x0 },
@@ -240,8 +260,12 @@ int default_buffered_input = -1;
 int read_from_stdin;		/* -s flag supplied */
 int want_pending_command;	/* -c flag supplied */
 
+/* This variable is not static so it can be bound to $BASH_EXECUTION_STRING */
+char *command_execution_string;	/* argument to -c option */
+
+int malloc_trace_at_exit = 0;
+
 static int shell_reinitialized = 0;
-static char *local_pending_command;
 
 static FILE *default_input;
 
@@ -252,6 +276,8 @@ static int parse_long_options __P((char **, int, int));
 static int parse_shell_options __P((char **, int, int));
 static int bind_args __P((char **, int, int, int));
 
+static void start_debugger __P((void));
+
 static void add_shopt_to_alist __P((char *, int));
 static void run_shopt_alist __P((void));
 
@@ -263,7 +289,6 @@ static int run_one_command __P((char *));
 static int run_wordexp __P((char *));
 
 static int uidget __P((void));
-static int isnetconn __P((int));
 
 static void init_interactive __P((void));
 static void init_noninteractive __P((void));
@@ -281,11 +306,11 @@ _cygwin32_check_tmp ()
   struct stat sb;
 
   if (stat ("/tmp", &sb) < 0)
-    internal_warning ("could not find /tmp, please create!");
+    internal_warning (_("could not find /tmp, please create!"));
   else
     {
       if (S_ISDIR (sb.st_mode) == 0)
-	internal_warning ("/tmp must be a valid directory name");
+	internal_warning (_("/tmp must be a valid directory name"));
     }
 }
 #endif /* __CYGWIN__ */
@@ -330,12 +355,8 @@ main (argc, argv, env)
   if (code)
     exit (2);
 
-#if defined (USING_BASH_MALLOC) && defined (DEBUG)
-#  if 0		/* memory tracing */
-  malloc_set_trace(1);
-#  endif
-
-#  if 0
+#if defined (USING_BASH_MALLOC) && defined (DEBUG) && !defined (DISABLE_MALLOC_WRAPPERS)
+#  if 1
   malloc_set_register (1);
 #  endif
 #endif
@@ -347,7 +368,7 @@ main (argc, argv, env)
 #endif /* __CYGWIN__ */
 
   /* Wait forever if we are debugging a login shell. */
-  while (debugging_login_shell);
+  while (debugging_login_shell) sleep (3);
 
   set_default_locale ();
 
@@ -372,7 +393,9 @@ main (argc, argv, env)
 
   /* Initialize `local' variables for all `invocations' of main (). */
   arg_index = 1;
-  local_pending_command = (char *)NULL;
+  if (arg_index > argc)
+    arg_index = argc;
+  command_execution_string = (char *)NULL;
   want_pending_command = locally_skip_execution = read_from_stdin = 0;
   default_input = stdin;
 #if defined (BUFFERED_INPUT)
@@ -417,7 +440,12 @@ main (argc, argv, env)
       exit (EXECUTION_SUCCESS);
     }
 
-  /* If user supplied the "--login" flag, then set and invert LOGIN_SHELL. */
+  /* All done with full word options; do standard shell option parsing.*/
+  this_command_name = shell_name;	/* for error reporting */
+  arg_index = parse_shell_options (argv, arg_index, argc);
+
+  /* If user supplied the "--login" (or -l) flag, then set and invert
+     LOGIN_SHELL. */
   if (make_login_shell)
     {
       login_shell++;
@@ -426,10 +454,6 @@ main (argc, argv, env)
 
   set_login_shell (login_shell != 0);
 
-  /* All done with full word options; do standard shell option parsing.*/
-  this_command_name = shell_name;	/* for error reporting */
-  arg_index = parse_shell_options (argv, arg_index, argc);
-
   if (dump_po_strings)
     dump_translatable_strings = 1;
 
@@ -444,32 +468,34 @@ main (argc, argv, env)
      following args are $0...$n respectively. */
   if (want_pending_command)
     {
-      local_pending_command = argv[arg_index];
-      if (local_pending_command == 0)
+      command_execution_string = argv[arg_index];
+      if (command_execution_string == 0)
 	{
-	  report_error ("option `-c' requires an argument");
-	  exit (EX_USAGE);
+	  report_error (_("%s: option requires an argument"), "-c");
+	  exit (EX_BADUSAGE);
 	}
       arg_index++;
     }
   this_command_name = (char *)NULL;
 
+  cmd_init();		/* initialize the command object caches */
+
   /* First, let the outside world know about our interactive status.
      A shell is interactive if the `-i' flag was given, or if all of
      the following conditions are met:
 	no -c command
 	no arguments remaining or the -s flag given
 	standard input is a terminal
-	standard output is a terminal
+	standard error is a terminal
      Refer to Posix.2, the description of the `sh' utility. */
 
   if (forced_interactive ||		/* -i flag */
-      (!local_pending_command &&	/* No -c command and ... */
+      (!command_execution_string &&	/* No -c command and ... */
        wordexp_only == 0 &&		/* No --wordexp and ... */
        ((arg_index == argc) ||		/*   no remaining args or... */
 	read_from_stdin) &&		/*   -s flag with args, and */
        isatty (fileno (stdin)) &&	/* Input is a terminal and */
-       isatty (fileno (stdout))))	/* output is a terminal. */
+       isatty (fileno (stderr))))	/* error output is a terminal. */
     init_interactive ();
   else
     init_noninteractive ();
@@ -490,11 +516,11 @@ main (argc, argv, env)
     }
 #endif /* CLOSE_FDS_AT_LOGIN */
 
-  /* If we're in a strict Posix.2 mode, turn on interactive comments and
-     other Posix.2 things. */
+  /* If we're in a strict Posix.2 mode, turn on interactive comments,
+     alias expansion in non-interactive shells, and other Posix.2 things. */
   if (posixly_correct)
     {
-      bind_variable ("POSIXLY_CORRECT", "y");
+      bind_variable ("POSIXLY_CORRECT", "y", 0);
       sv_strict_posix ("POSIXLY_CORRECT");
     }
 
@@ -510,13 +536,19 @@ main (argc, argv, env)
 
   if (interactive_shell)
     {
-      char *term;
+      char *term, *emacs;
 
-      term = getenv ("TERM");
+      term = get_string_value ("TERM");
       no_line_editing |= term && (STREQ (term, "emacs"));
-      term = getenv ("EMACS");
-      running_under_emacs = term ? ((strmatch ("*term*", term, 0) == 0) ? 2 : 1)
-				 : 0;
+      emacs = get_string_value ("EMACS");
+      running_under_emacs = emacs ? ((strstr (emacs, "term") != 0) ? 2 : 1) : 0;
+#if 0
+      no_line_editing |= emacs && emacs[0] == 't' && emacs[1] == '\0';
+#else
+      no_line_editing |= emacs && emacs[0] == 't' && emacs[1] == '\0' && STREQ (term, "dumb");
+#endif
+      if (running_under_emacs)
+	gnu_error_format = 1;
     }
 
   top_level_arg_index = arg_index;
@@ -528,7 +560,7 @@ main (argc, argv, env)
   code = setjmp (top_level);
   if (code)
     {
-      if (code == EXITPROG)
+      if (code == EXITPROG || code == ERREXIT)
 	exit_shell (last_command_exit_value);
       else
 	{
@@ -549,10 +581,13 @@ main (argc, argv, env)
 
   if (interactive_shell == 0)
     {
-      makunbound ("PS1", shell_variables);
-      makunbound ("PS2", shell_variables);
+      unbind_variable ("PS1");
+      unbind_variable ("PS2");
       interactive = 0;
+#if 0
+      /* This has already been done by init_noninteractive */
       expand_aliases = posixly_correct;
+#endif
     }
   else
     {
@@ -585,7 +620,7 @@ main (argc, argv, env)
   /* If we are invoked as `sh', turn on Posix mode. */
   if (act_like_sh)
     {
-      bind_variable ("POSIXLY_CORRECT", "y");
+      bind_variable ("POSIXLY_CORRECT", "y", 0);
       sv_strict_posix ("POSIXLY_CORRECT");
     }
 
@@ -605,16 +640,20 @@ main (argc, argv, env)
       exit_shell (last_command_exit_value);
     }
 
-  if (local_pending_command)
+  if (command_execution_string)
     {
       arg_index = bind_args (argv, arg_index, argc, 0);
       startup_state = 2;
+
+      if (debugging_mode)
+	start_debugger ();
+
 #if defined (ONESHOT)
       executing = 1;
-      run_one_command (local_pending_command);
+      run_one_command (command_execution_string);
       exit_shell (last_command_exit_value);
 #else /* ONESHOT */
-      with_input_from_string (local_pending_command, "-c");
+      with_input_from_string (command_execution_string, "-c");
       goto read_and_execute;
 #endif /* !ONESHOT */
     }
@@ -639,6 +678,10 @@ main (argc, argv, env)
 
   /* Bind remaining args to $1 ... $n */
   arg_index = bind_args (argv, arg_index, argc, 1);
+
+  if (debugging_mode && locally_skip_execution == 0 && running_setuid == 0)
+    start_debugger ();
+
   /* Do the things that should be done only for interactive shells. */
   if (interactive_shell)
     {
@@ -649,7 +692,10 @@ main (argc, argv, env)
 #if defined (HISTORY)
       /* Initialize the interactive history stuff. */
       bash_initialize_history ();
-      if (shell_initialized == 0)
+      /* Don't load the history from the history file if we've already
+	 saved some lines in this session (e.g., by putting `history -s xx'
+	 into one of the startup files). */
+      if (shell_initialized == 0 && history_lines_this_session == 0)
 	load_history ();
 #endif /* HISTORY */
 
@@ -698,9 +744,8 @@ parse_long_options (argv, arg_start, arg_end)
 		*long_args[i].int_value = 1;
 	      else if (argv[++arg_index] == 0)
 		{
-		  report_error ("option `%s' requires an argument",
-				long_args[i].name);
-		  exit (EX_USAGE);
+		  report_error (_("%s: option requires an argument"), long_args[i].name);
+		  exit (EX_BADUSAGE);
 		}
 	      else
 		*long_args[i].char_value = argv[arg_index];
@@ -712,9 +757,9 @@ parse_long_options (argv, arg_start, arg_end)
 	{
 	  if (longarg)
 	    {
-	      report_error ("%s: unrecognized option", argv[arg_index]);
+	      report_error (_("%s: invalid option"), argv[arg_index]);
 	      show_shell_usage (stderr, 0);
-	      exit (EX_USAGE);
+	      exit (EX_BADUSAGE);
 	    }
 	  break;		/* No such argument.  Maybe flag arg. */
 	}
@@ -759,6 +804,10 @@ parse_shell_options (argv, arg_start, arg_end)
 	      want_pending_command = 1;
 	      break;
 
+	    case 'l':
+	      make_login_shell = 1;
+	      break;
+
 	    case 's':
 	      read_from_stdin = 1;
 	      break;
@@ -771,7 +820,7 @@ parse_shell_options (argv, arg_start, arg_end)
 		  break;
 		}
 	      if (set_minus_o_option (on_or_off, o_option) != EXECUTION_SUCCESS)
-		exit (EX_USAGE);
+		exit (EX_BADUSAGE);
 	      next_arg++;
 	      break;
 
@@ -797,9 +846,9 @@ parse_shell_options (argv, arg_start, arg_end)
 	    default:
 	      if (change_flag (arg_character, on_or_off) == FLAG_ERROR)
 		{
-		  report_error ("%c%c: unrecognized option", on_or_off, arg_character);
+		  report_error (_("%c%c: invalid option"), on_or_off, arg_character);
 		  show_shell_usage (stderr, 0);
-		  exit (EX_USAGE);
+		  exit (EX_BADUSAGE);
 		}
 	    }
 	}
@@ -845,19 +894,22 @@ exit_shell (s)
 #endif /* JOB_CONTROL */
 
   /* Always return the exit status of the last command to our parent. */
-  exit (s);
+  sh_exit (s);
 }
 
-#ifdef INCLUDE_UNUSED
 /* A wrapper for exit that (optionally) can do other things, like malloc
    statistics tracing. */
 void
 sh_exit (s)
      int s;
 {
+#if defined (MALLOC_DEBUG) && defined (USING_BASH_MALLOC)
+  if (malloc_trace_at_exit)
+    trace_malloc_stats (get_name_for_error (), (char *)NULL);
+#endif
+
   exit (s);
 }
-#endif
 
 /* Source the bash startup files.  If POSIXLY_CORRECT is non-zero, we obey
    the Posix.2 startup file rules:  $ENV is expanded, and if the file it
@@ -911,7 +963,7 @@ run_startup_files ()
 
   /* get the rshd/sshd case out of the way first. */
   if (interactive_shell == 0 && no_rc == 0 && login_shell == 0 &&
-      act_like_sh == 0 && local_pending_command)
+      act_like_sh == 0 && command_execution_string)
     {
 #ifdef SSH_SOURCE_BASHRC
       run_by_ssh = (find_variable ("SSH_CLIENT") != (SHELL_VAR *)0) ||
@@ -943,8 +995,8 @@ run_startup_files ()
 
   sourced_login = 0;
 
-  /* A shell begun with the --login flag that is not in posix mode runs
-     the login shell startup files, no matter whether or not it is
+  /* A shell begun with the --login (or -l) flag that is not in posix mode
+     runs the login shell startup files, no matter whether or not it is
      interactive.  If NON_INTERACTIVE_LOGIN_SHELLS is defined, run the
      startup files if argv[0][0] == '-' as well. */
 #if defined (NON_INTERACTIVE_LOGIN_SHELLS)
@@ -1047,6 +1099,8 @@ shell_is_restricted (name)
   if (restricted)
     return 1;
   temp = base_pathname (name);
+  if (*temp == '-')
+    temp++;
   return (STREQ (temp, RESTRICTED_SHELL_NAME));
 }
 
@@ -1063,7 +1117,9 @@ maybe_make_restricted (name)
 {
   char *temp;
 
-  temp = base_pathname (shell_name);
+  temp = base_pathname (name);
+  if (*temp == '-')
+    temp++;
   if (restricted || (STREQ (temp, RESTRICTED_SHELL_NAME)))
     {
       set_var_read_only ("PATH");
@@ -1115,7 +1171,7 @@ run_wordexp (words)
      char *words;
 {
   int code, nw, nb;
-  WORD_LIST *wl, *result;
+  WORD_LIST *wl, *tl, *result;
 
   code = setjmp (top_level);
 
@@ -1126,6 +1182,7 @@ run_wordexp (words)
 	  /* Some kind of throw to top_level has occured. */
 	case FORCE_EOF:
 	  return last_command_exit_value = 127;
+	case ERREXIT:
 	case EXITPROG:
 	  return last_command_exit_value;
 	case DISCARD:
@@ -1149,6 +1206,9 @@ run_wordexp (words)
       if (global_command->type != cm_simple)
 	return (126);
       wl = global_command->value.Simple->words;
+      if (protected_mode)
+	for (tl = wl; tl; tl = tl->next)
+	  tl->word->flags |= W_NOCOMSUB;
       result = wl ? expand_words_no_vars (wl) : (WORD_LIST *)0;
     }
   else
@@ -1199,6 +1259,7 @@ run_one_command (command)
 	  /* Some kind of throw to top_level has occured. */
 	case FORCE_EOF:
 	  return last_command_exit_value = 127;
+	case ERREXIT:
 	case EXITPROG:
 	  return last_command_exit_value;
 	case DISCARD:
@@ -1219,7 +1280,7 @@ bind_args (argv, arg_start, arg_end, start_index)
   register int i;
   WORD_LIST *args;
 
-  for (i = arg_start, args = (WORD_LIST *)NULL; i != arg_end; i++)
+  for (i = arg_start, args = (WORD_LIST *)NULL; i < arg_end; i++)
     args = make_word_list (make_word (argv[i]), args);
   if (args)
     {
@@ -1232,9 +1293,13 @@ bind_args (argv, arg_start, arg_end, start_index)
 	  FREE (dollar_vars[0]);
 	  dollar_vars[0] = savestring (args->word->word);
 	  remember_args (args->next, 1);
+	  push_args (args->next);	/* BASH_ARGV and BASH_ARGC */
 	}
       else			/* bind to $1...$n for shell script */
-	remember_args (args, 1);
+        {
+	  remember_args (args, 1);
+	  push_args (args);		/* BASH_ARGV and BASH_ARGC */
+        }
 
       dispose_words (args);
     }
@@ -1246,6 +1311,23 @@ void
 unbind_args ()
 {
   remember_args ((WORD_LIST *)NULL, 1);
+  pop_args ();				/* Reset BASH_ARGV and BASH_ARGC */
+}
+
+static void
+start_debugger ()
+{
+#if defined (DEBUGGER) && defined (DEBUGGER_START_FILE)
+  int old_errexit;
+
+  old_errexit = exit_immediately_on_error;
+  exit_immediately_on_error = 0;
+
+  maybe_execute_file (DEBUGGER_START_FILE, 1);
+  function_trace_mode = 1;
+
+  exit_immediately_on_error += old_errexit;
+#endif
 }
 
 static int
@@ -1253,13 +1335,15 @@ open_shell_script (script_name)
      char *script_name;
 {
   int fd, e, fd_is_tty;
-  char *filename, *path_filename;
+  char *filename, *path_filename, *t;
   char sample[80];
   int sample_len;
   struct stat sb;
+#if defined (ARRAY_VARS)
+  SHELL_VAR *funcname_v, *bash_source_v, *bash_lineno_v;
+  ARRAY *funcname_a, *bash_source_a, *bash_lineno_a;
+#endif
 
-  free (dollar_vars[0]);
-  dollar_vars[0] = savestring (script_name);
   filename = savestring (script_name);
 
   fd = open (filename, O_RDONLY);
@@ -1286,6 +1370,24 @@ open_shell_script (script_name)
       exit ((e == ENOENT) ? EX_NOTFOUND : EX_NOINPUT);
     }
 
+  free (dollar_vars[0]);
+  dollar_vars[0] = savestring (script_name);
+
+#if defined (ARRAY_VARS)
+  GET_ARRAY_FROM_VAR ("FUNCNAME", funcname_v, funcname_a);
+  GET_ARRAY_FROM_VAR ("BASH_SOURCE", bash_source_v, bash_source_a);
+  GET_ARRAY_FROM_VAR ("BASH_LINENO", bash_lineno_v, bash_lineno_a);
+
+  array_push (bash_source_a, filename);
+  if (bash_lineno_a)
+    {
+      t = itos (executing_line_number ());
+      array_push (bash_lineno_a, t);
+      free (t);
+    }
+  array_push (funcname_a, "main");
+#endif
+
 #ifdef HAVE_DEV_FD
   fd_is_tty = isatty (fd);
 #else
@@ -1303,7 +1405,7 @@ open_shell_script (script_name)
 	{
 	  e = errno;
 	  if ((fstat (fd, &sb) == 0) && S_ISDIR (sb.st_mode))
-	    internal_error ("%s: is a directory", filename);
+	    internal_error (_("%s: is a directory"), filename);
 	  else
 	    {
 	      errno = e;
@@ -1386,17 +1488,12 @@ set_bash_input ()
   /* with_input_from_stdin really means `with_input_from_readline' */
   if (interactive && no_line_editing == 0)
     with_input_from_stdin ();
-  else
 #if defined (BUFFERED_INPUT)
-    {
-      if (interactive == 0)
-	with_input_from_buffered_stream (default_buffered_input, dollar_vars[0]);
-      else
-	with_input_from_stream (default_input, dollar_vars[0]);
-    }
-#else /* !BUFFERED_INPUT */
+  else if (interactive == 0)
+    with_input_from_buffered_stream (default_buffered_input, dollar_vars[0]);
+#endif /* BUFFERED_INPUT */
+  else
     with_input_from_stream (default_input, dollar_vars[0]);
-#endif /* !BUFFERED_INPUT */
 }
 
 /* Close the current shell script input source and forget about it.  This is
@@ -1434,17 +1531,7 @@ set_shell_name (argv0)
 {
   /* Here's a hack.  If the name of this shell is "sh", then don't do
      any startup files; just try to be more like /bin/sh. */
-  shell_name = base_pathname (argv0);
-  if (*shell_name == '-')
-    shell_name++;
-  if (shell_name[0] == 's' && shell_name[1] == 'h' && shell_name[2] == '\0')
-    act_like_sh++;
-  if (shell_name[0] == 's' && shell_name[1] == 'u' && shell_name[2] == '\0')
-    su_shell++;
-
-  shell_name = argv0;
-  FREE (dollar_vars[0]);
-  dollar_vars[0] = savestring (shell_name);
+  shell_name = argv0 ? base_pathname (argv0) : PROGRAM;
 
   if (*shell_name == '-')
     {
@@ -1452,6 +1539,15 @@ set_shell_name (argv0)
       login_shell++;
     }
 
+  if (shell_name[0] == 's' && shell_name[1] == 'h' && shell_name[2] == '\0')
+    act_like_sh++;
+  if (shell_name[0] == 's' && shell_name[1] == 'u' && shell_name[2] == '\0')
+    su_shell++;
+
+  shell_name = argv0 ? argv0 : PROGRAM;
+  FREE (dollar_vars[0]);
+  dollar_vars[0] = savestring (shell_name);
+
   /* A program may start an interactive shell with
 	  "execl ("/bin/bash", "-", NULL)".
      If so, default the name of this shell to our name. */
@@ -1473,7 +1569,7 @@ init_noninteractive ()
   bash_history_reinit (0);
 #endif /* HISTORY */
   interactive_shell = startup_state = interactive = 0;
-  expand_aliases = 0;
+  expand_aliases = posixly_correct;	/* XXX - was 0 not posixly_correct */
   no_line_editing = 1;
 #if defined (JOB_CONTROL)
   set_job_control (0);
@@ -1499,7 +1595,8 @@ get_current_user_info ()
 	}
       else
 	{
-	  current_user.user_name = savestring ("I have no name!");
+	  current_user.user_name = _("I have no name!");
+	  current_user.user_name = savestring (current_user.user_name);
 	  current_user.shell = savestring ("/bin/sh");
 	  current_user.home_dir = savestring ("/");
 	}
@@ -1530,7 +1627,7 @@ shell_initialize ()
      for restoring the original default signal handlers.  That function
      is called when we make a new child. */
   initialize_traps ();
-  initialize_signals ();
+  initialize_signals (0);
 
   /* It's highly unlikely that this will change. */
   if (current_host_name == 0)
@@ -1560,17 +1657,14 @@ shell_initialize ()
   initialize_shell_variables (shell_environment, privileged_mode||running_setuid);
 #endif
 
-#if 0
-  /* Initialize filename hash tables. */
-  initialize_filename_hashing ();
-#endif
-
   /* Initialize the data structures for storing and running jobs. */
   initialize_job_control (0);
 
   /* Initialize input streams to null. */
   initialize_bash_input ();
 
+  initialize_flags ();
+
   /* Initialize the shell options.  Don't import the shell options
      from the environment variable $SHELLOPTS if we are running in
      privileged or restricted mode or if the shell is running setuid. */
@@ -1620,7 +1714,7 @@ shell_reinitialize ()
 
   /* Delete all variables and functions.  They will be reinitialized when
      the environment is parsed. */
-  delete_all_variables (shell_variables);
+  delete_all_contexts (shell_variables);
   delete_all_variables (shell_functions);
 
   shell_reinitialized = 1;
@@ -1636,37 +1730,37 @@ show_shell_usage (fp, extra)
 
   if (extra)
     fprintf (fp, "GNU bash, version %s-(%s)\n", shell_version_string (), MACHTYPE);
-  fprintf (fp, "Usage:\t%s [GNU long option] [option] ...\n\t%s [GNU long option] [option] script-file ...\n",
+  fprintf (fp, _("Usage:\t%s [GNU long option] [option] ...\n\t%s [GNU long option] [option] script-file ...\n"),
 	     shell_name, shell_name);
-  fputs ("GNU long options:\n", fp);
+  fputs (_("GNU long options:\n"), fp);
   for (i = 0; long_args[i].name; i++)
     fprintf (fp, "\t--%s\n", long_args[i].name);
 
-  fputs ("Shell options:\n", fp);
-  fputs ("\t-irsD or -c command or -O shopt_option\t\t(invocation only)\n", fp);
+  fputs (_("Shell options:\n"), fp);
+  fputs (_("\t-irsD or -c command or -O shopt_option\t\t(invocation only)\n"), fp);
 
   for (i = 0, set_opts = 0; shell_builtins[i].name; i++)
     if (STREQ (shell_builtins[i].name, "set"))
       set_opts = savestring (shell_builtins[i].short_doc);
   if (set_opts)
     {
-      s = strchr (set_opts, '[');
+      s = xstrchr (set_opts, '[');
       if (s == 0)
 	s = set_opts;
       while (*++s == '-')
 	;
-      t = strchr (s, ']');
+      t = xstrchr (s, ']');
       if (t)
 	*t = '\0';
-      fprintf (fp, "\t-%s or -o option\n", s);
+      fprintf (fp, _("\t-%s or -o option\n"), s);
       free (set_opts);
     }
 
   if (extra)
     {
-      fprintf (fp, "Type `%s -c \"help set\"' for more information about shell options.\n", shell_name);
-      fprintf (fp, "Type `%s -c help' for more information about shell builtin commands.\n", shell_name);
-      fprintf (fp, "Use the `bashbug' command to report bugs.\n");
+      fprintf (fp, _("Type `%s -c \"help set\"' for more information about shell options.\n"), shell_name);
+      fprintf (fp, _("Type `%s -c help' for more information about shell builtin commands.\n"), shell_name);
+      fprintf (fp, _("Use the `bashbug' command to report bugs.\n"));
     }
 }
 
@@ -1692,56 +1786,8 @@ run_shopt_alist ()
 
   for (i = 0; i < shopt_ind; i++)
     if (shopt_setopt (shopt_alist[i].word, (shopt_alist[i].token == '-')) != EXECUTION_SUCCESS)
-      exit (EX_USAGE);
+      exit (EX_BADUSAGE);
   free (shopt_alist);
   shopt_alist = 0;
   shopt_ind = shopt_len = 0;
 }
-
-/* The second and subsequent conditions must match those used to decide
-   whether or not to call getpeername() in isnetconn(). */
-#if defined (HAVE_SYS_SOCKET_H) && defined (HAVE_GETPEERNAME) && !defined (SVR4_2)
-#  include <sys/socket.h>
-#endif
-
-/* Is FD a socket or network connection? */
-static int
-isnetconn (fd)
-     int fd;
-{
-#if defined (HAVE_GETPEERNAME) && !defined (SVR4_2) && !defined (__BEOS__)
-  int rv;
-  socklen_t l;
-  struct sockaddr sa;
-
-  l = sizeof(sa);
-  rv = getpeername(fd, &sa, &l);
-  /* Solaris 2.5 getpeername() returns EINVAL if the fd is not a socket. */
-  return ((rv < 0 && (errno == ENOTSOCK || errno == EINVAL)) ? 0 : 1);
-#else /* !HAVE_GETPEERNAME || SVR4_2 || __BEOS__ */
-#  if defined (SVR4) || defined (SVR4_2)
-  /* Sockets on SVR4 and SVR4.2 are character special (streams) devices. */
-  struct stat sb;
-
-  if (isatty (fd))
-    return (0);
-  if (fstat (fd, &sb) < 0)
-    return (0);
-#    if defined (S_ISFIFO)
-  if (S_ISFIFO (sb.st_mode))
-    return (0);
-#    endif /* S_ISFIFO */
-  return (S_ISCHR (sb.st_mode));
-#  else /* !SVR4 && !SVR4_2 */
-#    if defined (S_ISSOCK) && !defined (__BEOS__)
-  struct stat sb;
-
-  if (fstat (fd, &sb) < 0)
-    return (0);
-  return (S_ISSOCK (sb.st_mode));
-#    else /* !S_ISSOCK || __BEOS__ */
-  return (0);
-#    endif /* !S_ISSOCK || __BEOS__ */
-#  endif /* !SVR4 && !SVR4_2 */
-#endif /* !HAVE_GETPEERNAME || SVR4_2 || __BEOS__ */
-}
