@@ -1,6 +1,6 @@
 /* shell.h -- The data structures used by the shell */
 
-/* Copyright (C) 1993 Free Software Foundation, Inc.
+/* Copyright (C) 1993-2002 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
@@ -18,7 +18,9 @@
    with Bash; see the file COPYING.  If not, write to the Free Software
    Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA. */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 
 #include "bashjmp.h"
 
@@ -27,11 +29,13 @@
 #include "general.h"
 #include "error.h"
 #include "variables.h"
+#include "arrayfunc.h"
 #include "quit.h"
 #include "maxpath.h"
 #include "unwind_prot.h"
 #include "dispose_cmd.h"
 #include "make_cmd.h"
+#include "ocache.h"
 #include "subst.h"
 #include "sig.h"
 #include "pathnames.h"
@@ -41,7 +45,6 @@ extern int EOF_Reached;
 
 #define NO_PIPE -1
 #define REDIRECT_BOTH -2
-#define IS_DESCRIPTOR -1
 
 #define NO_VARIABLE -1
 
@@ -67,26 +70,30 @@ extern int EOF_Reached;
 #define EX_EXPFAIL	261	/* word expansion failed */
 
 /* Flag values that control parameter pattern substitution. */
-#define MATCH_ANY	0x0
-#define MATCH_BEG	0x1
-#define MATCH_END	0x2
+#define MATCH_ANY	0x000
+#define MATCH_BEG	0x001
+#define MATCH_END	0x002
 
-#define MATCH_TYPEMASK	0x3
+#define MATCH_TYPEMASK	0x003
 
-#define MATCH_GLOBREP	0x10
-#define MATCH_QUOTED	0x20
+#define MATCH_GLOBREP	0x010
+#define MATCH_QUOTED	0x020
+#define MATCH_STARSUB	0x040
 
 /* Some needed external declarations. */
 extern char **shell_environment;
 extern WORD_LIST *rest_of_args;
 
 /* Generalized global variables. */
+extern int debugging_mode;
 extern int executing, login_shell;
+extern int interactive, interactive_shell;
+extern int startup_state;
 
 /* Structure to pass around that holds a bitmap of file descriptors
    to close, and the size of that structure.  Used in execute_cmd.c. */
 struct fd_bitmap {
-  long size;
+  int size;
   char *bitmap;
 };
 
@@ -105,3 +112,52 @@ struct user_info {
 };
 
 extern struct user_info current_user;
+
+/* Force gcc to not clobber X on a longjmp().  Old versions of gcc mangle
+   this badly. */
+#if (__GNUC__ > 2) || (__GNUC__ == 2 && __GNUC_MINOR__ > 8)
+#  define USE_VAR(x)	((void) &(x))
+#else
+#  define USE_VAR(x)
+#endif
+
+/* Structure in which to save partial parsing state when doing things like
+   PROMPT_COMMAND and bash_execute_unix_command execution. */
+
+typedef struct _sh_parser_state_t {
+
+  /* parsing state */
+  int parser_state;
+  int *token_state;
+
+  /* input line state -- line number saved elsewhere */
+  int input_line_terminator;
+  int eof_encountered;
+
+#if defined (HANDLE_MULTIBYTE)
+  /* Nothing right now for multibyte state, but might want something later. */
+#endif
+
+  /* history state affecting or modified by the parser */
+  int current_command_line_count;
+#if defined (HISTORY)
+  int remember_on_history;
+  int history_expansion_inhibited;
+#endif
+
+  /* execution state possibly modified by the parser */
+  int last_command_exit_value;
+#if defined (ARRAY_VARS)
+  ARRAY *pipestatus;
+#endif
+  sh_builtin_func_t *last_shell_builtin, *this_shell_builtin;
+
+  /* flags state affecting the parser */
+  int expand_aliases;
+  int echo_input_at_read;
+  
+} sh_parser_state_t;
+
+/* Let's try declaring these here. */
+extern sh_parser_state_t *save_parser_state __P((sh_parser_state_t *));
+extern void restore_parser_state __P((sh_parser_state_t *));
